package com.milanpresse.engineaddons.facades
{
   import com.milanpresse.engine.Engine;
   import com.milanpresse.engine.config.Config;
   import com.milanpresse.engine.managers.PauseManager;
   import com.milanpresse.engine.managers.TimelineManager;
   import com.milanpresse.engine.medias.MediaImage;
   import com.milanpresse.engine.medias.MediaSon;
   import com.milanpresse.engine.medias.MediaSwf;
   import com.milanpresse.engine.medias.MediaTexte;
   import com.milanpresse.engine.text.TextBloc;
   import com.milanpresse.engine.typeargument.CommentObject;
   import com.milanpresse.engine.typeargument.NoiseObject;
   import com.milanpresse.engineaddons.gamesprotos.Actor;
   import com.milanpresse.engineaddons.gamesprotos.actors.actorClip.ActorClip;
   import com.milanpresse.engineaddons.gamesprotos.actors.actorClip.ActorTimer;
   import com.milanpresse.engineaddons.gamesprotos.actors.cursor.Cursor;
   import com.milanpresse.engineaddons.timers.MainTimer;
   import flash.display.DisplayObjectContainer;
   import flash.display.MovieClip;
   import flash.display.Sprite;
   import flash.display.Stage;
   import flash.net.URLRequest;
   import flash.net.navigateToURL;
   
   public class GameEngine
   {
      protected var _engine:Engine;
      
      public var config:Config;
      
      public var stage:Stage;
      
      public var timelineManager:TimelineManager;
      
      protected var actors:Vector.<Actor>;
      
      public var memo:Object;
      
      private var _pauseManager:PauseManager;
      
      private var mainActor:Actor;
      
      public var mainAlgo:*;
      
      public var mainTimer:MainTimer;
      
      private var classes:Array;
      
      public var moduleConfig:*;
      
      public var cursor:Cursor;
      
      public function GameEngine(param1:*)
      {
         this.timelineManager = new TimelineManager();
         this.actors = new Vector.<Actor>();
         this.memo = {};
         this.classes = [ActorClip,ActorTimer];
         this.moduleConfig = {};
         super();
         this._engine = param1.engine;
         this.config = this._engine.config;
         this.stage = this.config.ROOT_APP.stage;
         this.mainActor = new Actor(this);
         this.mainTimer = new MainTimer();
         this.init();
      }
      
      public function init() : void
      {
         this.levelCZ = this.config.layers.content;
      }
      
      public function set subtitles(param1:*) : void
      {
         if(param1 is Boolean)
         {
            this._engine.setSubtitles(param1);
         }
         else if(param1 is String && param1 == "auto")
         {
            this._engine.setSubtitles(this._engine.host.subtitlesOn);
         }
      }
      
      public function get subtitles() : Boolean
      {
         return this._engine.config.subtitlesOn;
      }
      
      public function set levelCZ(param1:Sprite) : void
      {
         this._engine.levelCZ = param1;
      }
      
      public function removeActor(param1:Actor) : void
      {
         this.mainActor.removeActor(param1);
      }
      
      public function getActor(param1:*) : *
      {
         return this.mainActor.getActor(param1);
      }
      
      public function addActor(param1:*) : *
      {
         return this.mainActor.addActor(param1);
      }
      
      public function foreach(param1:Function) : void
      {
         this.mainActor.foreach(param1);
      }
      
      public function ibHide() : void
      {
         this._engine.hideIB();
      }
      
      public function ibShow(param1:Object) : void
      {
         this._engine.showIB(param1);
      }
      
      public function cursorInit(param1:*) : void
      {
         var _loc2_:MovieClip = null;
         if(param1.mc != undefined)
         {
            _loc2_ = param1.mc;
         }
         else if(param1.id != undefined)
         {
            _loc2_ = (this._engine.getMedia(param1) as MediaSwf).swfContent as MovieClip;
         }
         else if(param1.code != undefined)
         {
            _loc2_ = (this._engine.getMediaSwf(param1.code) as MediaSwf).swfContent as MovieClip;
         }
         else if(param1.name != undefined)
         {
            _loc2_ = (this._engine.getMediaSwf(param1.code) as MediaSwf).swfContent as MovieClip;
         }
         if(param1.parent == undefined)
         {
            param1.parent = this.stage;
         }
         this.cursor = new Cursor(_loc2_,param1.parent);
      }
      
      private function getMediasWithSimilarPrefix(param1:String) : Array
      {
         var _loc2_:Array = this.config.mediasManager.medias;
         var _loc3_:int = int(_loc2_.length);
         var _loc4_:Array = [];
         var _loc5_:int = 0;
         while(_loc5_ < _loc3_)
         {
            if(_loc2_[_loc5_].id.search("^" + param1) != -1)
            {
               _loc4_.push(_loc2_[_loc5_]);
            }
            _loc5_++;
         }
         return _loc4_;
      }
      
      public function noisePlay(param1:*) : MediaSon
      {
         if(param1 is NoiseObject)
         {
            return this._engine.playNoise(param1);
         }
         param1.overdub = true;
         param1.id = this.config.noiseName + param1.code;
         return this._engine.playSound(param1);
      }
      
      public function noiseStop(param1:* = null) : void
      {
         if(param1 == null)
         {
            this._engine.stopAllNoiseExceptMusic();
         }
         else if(param1 is MediaSon)
         {
            if(param1 == null)
            {
               return;
            }
            param1.stop();
            this._engine.destroyMedia(param1.id);
         }
         else
         {
            this._engine.stopNoise(param1);
            this._engine.destroyMedia(this.config.noiseName + param1.code);
         }
      }
      
      public function openUrl(param1:String) : void
      {
         var url:String = param1;
         var request:URLRequest = new URLRequest(url);
         try
         {
            navigateToURL(request,"_blank");
         }
         catch(e:Error)
         {
         }
      }
      
      public function get engine() : Engine
      {
         return this._engine;
      }
      
      public function get framerate() : Number
      {
         return this.stage.frameRate;
      }
      
      public function set framerate(param1:Number) : void
      {
         this._engine.setFrameRate(param1);
      }
      
      public function playMusic(param1:Object) : void
      {
         this._engine.playMusic(param1);
      }
      
      public function getMediaImage(param1:String) : MediaImage
      {
         return this._engine.getMediaImage(param1);
      }
      
      public function showImage(param1:Object) : MediaImage
      {
         return this._engine.showImage(param1);
      }
      
      public function showTextBloc(param1:Object) : TextBloc
      {
         return this._engine.showTextBloc(param1);
      }
      
      public function getMediaText(param1:Object) : MediaTexte
      {
         return this._engine.config.texteManager.getMedias(param1);
      }
      
      public function commentPlay(param1:Object) : void
      {
         this._engine.playComment(new CommentObject(param1));
      }
      
      public function commentStop() : void
      {
         this._engine.stopComment();
         if(this.config.musicOn == true)
         {
            if(this.config.MusiqueFond !== null)
            {
               this.config.MusiqueFond.volume = 1;
            }
         }
      }
      
      public function commentStopAll() : void
      {
         this._engine.stopAllComments();
      }
      
      public function changeModule(param1:Object) : void
      {
         this._engine.host.changeModule(param1);
      }
      
      public function start() : void
      {
         if(this.mainAlgo != null)
         {
            this.mainAlgo.start();
         }
      }
      
      public function sleep(param1:DisplayObjectContainer = null, param2:Boolean = true) : void
      {
         if(this.mainAlgo != null)
         {
            this.mainAlgo.sleep();
         }
         if(this.mainActor != null)
         {
            this.mainActor.sleep();
         }
         if(this.mainTimer != null)
         {
            this.mainTimer.sleep();
         }
         this._pauseManager = new PauseManager(this._engine,this.timelineManager,param2);
         this._pauseManager.sleepMovie(param1);
      }
      
      public function wake(param1:DisplayObjectContainer = null) : void
      {
         this._pauseManager.wakeMovie(param1);
         if(this.mainAlgo != null)
         {
            this.mainAlgo.wake();
         }
         if(this.mainActor != null)
         {
            this.mainActor.wake();
         }
         if(this.mainTimer != null)
         {
            this.mainTimer.wake();
         }
      }
      
      public function destroy() : void
      {
         this.sleep();
         if(this.mainAlgo != null)
         {
            this.mainAlgo.destroy();
         }
         if(this.mainActor != null)
         {
            this.mainActor.destroy();
            this.mainActor = null;
         }
         if(this.mainTimer != null)
         {
            this.mainTimer.destroy();
         }
         this._engine.destroy();
      }
      
      public function get inHost() : Boolean
      {
         return this.config.APP != this.config.ROOT_APP;
      }
   }
}

